@@ -1,15 +1,17 @@
 from canalyst_candas.utils import (
-    get_api_headers,
     Getter,
     LogFile,
     SCENARIO_URL,
+    calendar_quarter,
     crawl_company_pages,
     create_drivers_dot,
     df_filter,
     dot_parse,
     filter_dataset,
+    get_api_headers,
     get_company_info_from_ticker,
-    get_drivers_from_model,
+    get_drivers_from_api,
+    get_name_index_from_csv,
     get_forecast_url,
     get_forecast_url_data,
     get_model_info,
@@ -67,9 +69,13 @@ from pathlib import Path
 
 from canalyst_candas.configuration.config import Config
 from canalyst_candas.settings import CONFIG
+from canalyst_candas import settings
 
 
 def help():
+    """
+    Canalyst Candas help function
+    """
     print("Canalyst Candas help")
     print(
         "Please go to https://pypi.org/project/canalyst-candas/#description for installation help"
@@ -79,10 +85,14 @@ def help():
     print("For an Excel model download:get_excel_model(ticker, config)")
 
 
-# helper class for on the fly function generation used in the ForecastFrame concept
-# where we try to change one param across multiple tickers for re-fit in the
-# scenario engine
 class FuncMusic(object):
+    """
+    Helper class for on the fly function generation.
+
+    Used in the ForecastFrame concept where we try to change one param
+    across multiple tickers for re-fit in the scenario engine.
+    """
+
     def apply_function(self, value, modifier, argument):
         self.value = value
         self.modifier = modifier
@@ -123,10 +133,6 @@ class Search:
             ",",
             "%",
             "(Calculated)",
-            "Total",
-            "growth",
-            "Growth",
-            "Average",
             "(QoQ)",
             "QoQ",
             "mm",
@@ -171,6 +177,7 @@ class Search:
         file_name="",
         time_series_name="",
         time_series_description="",
+        query="",
         most_recent="",
     ):
         if self.df_guidance is None:
@@ -181,19 +188,20 @@ class Search:
             print("Done")
 
         df = self.df_guidance
+        df = df.loc[df["Type.1"] == "Estimate"]
 
-        df["time_series_description"] = df["Item"]
-        df["time_series_name"] = df["Item Name"]
-        df.drop(columns=["Item", "Item Name"])
+        df = df.rename(
+            columns={"Item": "time_series_description", "Item Name": "time_series_name"}
+        )
 
         if most_recent == True:
-            df = df.sort_values(["ticker", "Date"], ascending=False)
+            df = df.sort_values(["ticker", "time_series_name", "Date"], ascending=False)
             df = df.groupby(["ticker", "time_series_name"]).first().reset_index()
 
         if type(ticker) == list:
             df = df.loc[df["ticker"].isin(ticker)]
         elif ticker != "":
-            df = df.loc[df["ticker"].str.contains(ticker, case=True, regex=True)]
+            df = df.loc[df["ticker"] == ticker]
 
         if sector != "":
             df = df.loc[df["Path"].str.contains(sector, case=True, regex=True)]
@@ -203,19 +211,25 @@ class Search:
                     time_series_name, case=False, regex=True
                 )
             ]
+
         if time_series_description != "":
-            df = df.loc[
-                df["time_series_description"].str.contains(
-                    time_series_description, case=False
-                )
-            ]
+
+            ts_lower = time_series_description.lower()
+            split_strings = ts_lower.split(" ")
+            df["ts_lower"] = df["time_series_description"].str.lower()
+            df["search_matches"] = df["ts_lower"].str.count("|".join(split_strings))
+            df = df.loc[df["search_matches"] > 0]
+
+        else:
+            df["search_matches"] = 1
 
         if file_name != "":
             df = df.loc[df["Filename"].str.contains(file_name, case=False)]
 
-        return df[
+        df = df[
             [
                 "ticker",
+                "CSIN",
                 "Path",
                 "Filename",
                 "time_series_description",
@@ -227,8 +241,16 @@ class Search:
                 "Type.1",
                 "Date",
                 "Link",
+                "search_matches",
             ]
-        ].sort_values(["ticker", "Date", "time_series_description"])
+        ].sort_values(
+            ["search_matches", "ticker", "Date", "time_series_name"], ascending=False
+        )
+
+        if query != "":
+            df = df.query(query)
+            df = df.sort_values(["search_matches", "value"], ascending=False)
+        return df
 
     def match_time_series(self, ticker="", time_series=""):
         if type(time_series) == "str":
@@ -291,10 +313,10 @@ class Search:
 
             ts_lower = time_series_description.lower()
             split_strings = ts_lower.split(" ")
-
             df["ts_lower"] = df["time_series_description"].str.lower()
             df["search_matches"] = df["ts_lower"].str.count("|".join(split_strings))
             df = df.loc[df["search_matches"] > 0]
+
         else:
             df["search_matches"] = 1
 
@@ -314,6 +336,7 @@ class Search:
         df = df[
             [
                 "ticker",
+                "CSIN",
                 "Path",
                 "Filename",
                 "category",
@@ -330,7 +353,7 @@ class Search:
         )
         if query != "":
             df = df.query(query)
-            df = df.sort_values("value", ascending=False)
+            df = df.sort_values(["search_matches", "value"], ascending=False)
         return df
 
 
@@ -917,7 +940,6 @@ class ModelSet:
         df["ticker"] = ticker
         return df
 
-    # shows a tree for the given ticker and time series
     def create_model_map(
         self,
         ticker,
@@ -926,6 +948,9 @@ class ModelSet:
         tree=True,
         notebook=False,
     ):
+        """
+        Returns a tree for the given ticker and time series
+        """
         if type(ticker) == list:
             print("Please request one ticker at a time.")
             return
@@ -943,18 +968,120 @@ class ModelSet:
             print("Please choose a ticker in this ModelSet's ticker list")
             return
 
-    def create_time_series_chart(self, ticker, time_series_name):  # MODELSET
+    def create_time_series_chart(
+        self, ticker="", time_series_name="", historical=False
+    ):  # MODELSET
+        """
+        Create a time series chart for the given ticker and time series
+        """
         if type(ticker) == list:
             print("Please request one ticker at a time.")
             return
 
+        if ticker == "":
+            df = self._features
+
+            if type(time_series_name) == list:
+                df = df.loc[self._features["time_series_name"].isin(time_series_name)]
+            else:
+                df = df.loc[self._features["time_series_name"] == time_series_name]
+
+            if historical:
+                df = df.loc[df["is_historical"] == historical]
+
+            df = df.loc[df["period_name"].str.contains("Q")].sort_values(
+                ["ticker", "period_end_date"]
+            )
+            df = df.loc[df["period_end_date"] > "2015-01-01"]
+            df = df.dropna(subset=["value"])
+            # return df
+            # df = df.groupby('ticker').size().order(ascending=False).reset_index()
+
+            row1 = df.iloc[0]
+            # title = self.ticker + " " + str(row1["time_series_description"])
+            xlabel = "Fiscal Quarter"
+            # ylabel = row1["time_series_description"]
+
+            colors = {
+                "red": "#ff207c",
+                "grey": "#C3C2C3",
+                "blue": "#00838F",
+                "orange": "#ffa320",
+                "green": "#00ec8b",
+            }
+
+            plt.rc("figure", figsize=(12, 9))
+
+            if type(time_series_name) != list:
+
+                df_plot = df.loc[
+                    df["time_series_name"] == time_series_name
+                ].sort_values(["ticker", "time_series_name"])
+
+                if ticker == "":
+                    ticker_list = self.ticker_list
+                elif type(ticker) == list:
+                    ticker_list = ticker
+                else:
+                    ticker_list = [ticker]
+
+                for ticker in ticker_list:
+
+                    df_plot_one = df_plot[df_plot["ticker"] == ticker]
+                    df_plot_one = df_plot_one.sort_values("period_end_date")
+
+                    if df_plot_one.shape[0] > 0:
+                        plt.plot(
+                            df_plot_one["period_end_date"],
+                            df_plot_one["value"],
+                            label=ticker + " " + time_series_name,
+                            linewidth=2,
+                        )
+                        plt.axvline(
+                            x=self.models[ticker].mrq()["period_end_date"][0],
+                            color="black",
+                        )
+            else:
+
+                if ticker == "":
+                    ticker_list = self.ticker_list
+                elif type(ticker) == list:
+                    ticker_list = ticker
+                else:
+                    ticker_list = [ticker]
+
+                for tsn in time_series_name:
+                    df_plot = df.loc[df["time_series_name"] == tsn]
+                    for ticker in ticker_list:
+                        df_plot_one = df_plot[df_plot["ticker"] == ticker]
+                        df_plot_one = df_plot_one.sort_values("period_end_date")
+                        if df_plot_one.shape[0] > 0:
+                            plt.plot(
+                                df_plot_one["period_end_date"],
+                                df_plot_one["value"],
+                                label=ticker + " " + tsn,
+                                linewidth=2,
+                            )
+                            plt.axvline(
+                                x=self.models[ticker].mrq()["period_end_date"][0],
+                                color="black",
+                            )
+
+            plt.xticks(rotation=90)
+            plt.legend()
+            plt.show()
+            return
+
         if ticker in self.ticker_list:
-            self.models[ticker].create_time_series_chart(time_series_name)
+            self.models[ticker].create_time_series_chart(time_series_name, historical)
         else:
             print("Please choose a ticker in this ModelSet's ticker list")
         return
 
     def guidance(self, ticker):
+        """
+        Get guidance for ticker or list of tickers
+        """
         if type(ticker) == list:
             list_df = []
             for t in ticker:
@@ -979,6 +1106,9 @@ class ModelSet:
         return
 
     def mrq(self, ticker):
+        """
+        Gets the most recent quarter for a ticker or list of tickers
+        """
         if type(ticker) == list:
             list_df = []
             for t in ticker:
@@ -994,7 +1124,6 @@ class ModelSet:
                 print("Please choose a ticker in this ModelSet's ticker list")
         return
 
-    # get a list of time series from the default dataframe
     def time_series_names(self):
         df = self._features
         df = (
@@ -1005,8 +1134,12 @@ class ModelSet:
         self.FeatureList = df
         return
 
-    # search the time series for the default dataframe
     def time_series_search(self, search_term=""):
+        """
+        Searches for the search term in the default dataframe
+
+        Returns dataframe with columns ['time_series_name','time_series_description','category_slug']
+        """
         df = self._features
         if search_term != "":
             df1 = df[
@@ -1029,8 +1162,10 @@ class ModelSet:
             )
         return df  # [['time_series_name','time_series_description','category_slug']]
 
-    # search model drivers
     def driver_search(self, search_term="", col="time_series_name"):
+        """
+        Search model drivers
+        """
         df = self.drivers
         if search_term != "":
             df1 = df[
@@ -1054,10 +1189,15 @@ class ModelSet:
         return df  # [['time_series_name','time_series_description','category_slug']]
 
     def common_time_series(self):
+        """
+        Get common time series names
+        """
         return self._common_time_series_names
 
-    # creates the default dataframes features and Drivers
     def get_featurelibrary(self, col="time_series_name"):
+        """
+        Creates the default dataframe features and drivers
+        """
         if type(self.ticker_list) is list:
 
             list_all = []
@@ -1123,6 +1263,7 @@ class ModelSet:
         category="",
         warning=True,
         ticker="",
+        n_diff="",
     ):
 
         if ticker != "":
@@ -1144,6 +1285,7 @@ class ModelSet:
             unit_type,
             category,
             warning,
+            n_diff,
         )
 
     def modify_features(
@@ -1373,9 +1515,12 @@ class ModelMap:
 
         self.fig = None
 
-    # helper function to load data from candas:
-    # loads model_frame for chosen ticker and create a dot file
     def load_data(self) -> pd.DataFrame:
+        """
+        Helper function to load data from candas
+
+        Loads model_frame for chosen ticker and creates a dot file
+        """
         if self.model:
             cdm = self.model
         else:
@@ -1389,8 +1534,10 @@ class ModelMap:
 
         return df
 
-    # create a model of the dot file and optionally download in an html file
     def create_model(self, toggle_drag_nodes=True):
+        """
+        Create a model of the dot file and optionally download in an html file
+        """
         # read dot file into a graph
         G = nx.drawing.nx_pydot.read_dot(self.dot_file)
 
@@ -1584,12 +1731,16 @@ class ModelMap:
 
         return graph
 
-    # shows the model
     def show(self):
+        """
+        Shows the model map
+        """
         return self.network.show(self.model_path)
 
-    # Creates a dataframe of each nodes mo_name and distance from root
     def create_node_df(self):
+        """
+        Creates a dataframe of each node's MO_name and distance from root
+        """
         graph = self.network
 
         # root node id
@@ -1617,8 +1768,10 @@ class ModelMap:
 
         return node_df
 
-    # lists out all the nodes
     def list_time_series(self, search=""):
+        """
+        Lists out all the nodes
+        """
         list_out = []
         if self.nodes:
             for node in self.nodes:
@@ -1626,8 +1779,12 @@ class ModelMap:
                     list_out.append(node["title"].split(":")[0])
         return list_out
 
-    # lists out all the nodes - duplicate of above for backwards comp
     def time_series_names(self, search=""):
+        """
+        (Obsolete) Lists out all the nodes
+
+        This method is kept for backwards-compatibility. New code should use list_time_series instead.
+        """
         list_out = []
         if self.nodes:
             for node in self.nodes:
@@ -1635,8 +1792,10 @@ class ModelMap:
                     list_out.append(node["title"].split(":")[0])
         return list_out
 
-    # creates chart of given time series:
     def create_time_series_chart(self, time_series_name):
+        """
+        Create a chart of given time series
+        """
         # get needed data from dataframe
         df = self.df[self.df["time_series_name"] == time_series_name]
 
@@ -1662,8 +1821,10 @@ class ModelMap:
         )
         return fig
 
-    # shows a chart of the given time series
     def show_time_series_chart(self, time_series_name):
+        """
+        Shows a chart of the given time series
+        """
         fig = self.create_time_series_chart(time_series_name)
         fig.show()
         return
@@ -1809,7 +1970,9 @@ class Model:
         return
 
     def get_most_recent_model_date(self):
-
+        """
+        Get published date for latest workbook model
+        """
         csin = self.csin
         auth_headers = self.api_headers
         mds_host = self.config.mds_host
@@ -1857,7 +2020,9 @@ class Model:
         return date_name
 
     def get_excel_model_name(self):
-
+        """
+        Get the model workbook's file name
+        """
         csin = self.csin
         auth_headers = self.api_headers
         mds_host = self.config.mds_host
@@ -1904,6 +2069,9 @@ class Model:
         return file_name
 
     def key_driver_map(self, time_series_name):
+        """
+        Returns a dataframe of key drivers for the MRQ
+        """
         # defaulting to MO_RIS_REV, but you could choose another time_series_name as a starting point like MO_RIS_EBIT
         model_map = self.create_model_map(
             time_series_name=time_series_name,
@@ -1970,7 +2138,7 @@ class Model:
 
         return df
 
-    def create_time_series_chart(self, time_series_name):  # MODEL
+    def create_time_series_chart(self, time_series_name, historical=False):  # MODEL
         # get needed data from dataframe
         df = self._model_frame[
             self._model_frame["time_series_name"] == time_series_name
@@ -1979,6 +2147,9 @@ class Model:
             print("Time series not found")
             return None
 
+        if historical:
+            df = df.loc[df["is_historical"] == historical]
+
         df = df[df["period_name"].str.contains("Q")].sort_values("period_end_date")
         # use subset=['value'] to only drop rows with a null value
         df = df.dropna(subset=["value"])
@@ -2020,6 +2191,11 @@ class Model:
         notebook=False,
         common_time_series_names=None,
     ):
+        """
+        Create a model map from this model, rooted at the specified time series
+
+        Defaults to a model map rooted at MO_RIS_REV if no time series name is provided
+        """
         mm = ModelMap(
             config=self.config,
             model=self,
@@ -2085,6 +2261,9 @@ class Model:
         notebook=True,
         common_time_series=None,
     ):
+        """
+        Display model_map
+        """
         mm = ModelMap(
             config=self.config,
             model=self,
@@ -2097,6 +2276,11 @@ class Model:
         return mm.show()
 
     def create_guidance_csv(self):
+        """
+        Write a CSV file containing the model's guidance data
+
+        Parses guidance from the model's Excel workbook
+        """
         file_ticker = self.ticker.split(" ")[0]
         path_name = f"{self.config.default_dir}/DATA/{file_ticker}/"
         os.makedirs(path_name, exist_ok=True)
@@ -2144,39 +2328,49 @@ class Model:
         self._guidance = df
         return df
 
-    def get_driverlibrary(self, force=False):
+    def get_drivers(self, force=False):
+        """
+        Gets the associated list of drivers for the model.
+
+        Supply with force=False to use cache.
+        """
         df = self.gt.get_file(self.ticker, "model_drivers")
 
         if df is not None and force == False:
             self._model_drivers = df
             return
-        print("get drivers from model")
 
-        df_drivers, self.df_name_index = get_drivers_from_model(
-            self.ticker,
-            self.config,
-        )
-        df_drivers = df_drivers.loc[df_drivers["historical"] == 0]
-        df = (
-            df_drivers.groupby("named_range")
-            .first()
-            .reset_index()
-            .sort_values("cell")[["named_range"]]
+        drivers = get_drivers_from_api(
+            self.config.mds_host, self.csin, self.latest_version, self.api_headers
         )
 
+        df_new = pd.json_normalize(drivers)
+        df_driver_array = pd.DataFrame(df_new["time_series.names"])
+        df_drivers = df_driver_array.explode("time_series.names")
+        df = df_drivers.groupby("time_series.names").first().reset_index()
         df.columns = ["time_series_name"]
+
         self._model_drivers = df
 
         ticker = self.ticker.split(" ")[0]
+
         self._model_drivers.to_csv(
             f"{self.config.default_dir}/DATA/{ticker}/{self.ticker}_model_drivers.csv",
             index=False,
         )
+
+        self.get_name_index(force=self.force)
+
+    def get_name_index(self, force=False):
+
+        self.df_name_index = get_name_index_from_csv(self.ticker, self.config)
+
+        ticker = self.ticker.split(" ")[0]
+
         self.df_name_index.to_csv(
             f"{self.config.default_dir}/DATA/{ticker}/{self.ticker}_name_index.csv",
             index=False,
         )
-
         return
 
     def revenue_drivers(self, search_term="", is_driver=False):
@@ -2271,6 +2465,7 @@ class Model:
         unit_type="",
         category="",
         warning=True,
+        n_diff="",
     ):
         return filter_dataset(
             self._model_frame,
@@ -2298,7 +2493,7 @@ class Model:
             self._model_drivers = df
         else:
             print("get driver library")
-            self.get_driverlibrary(force=self.force)
+            self.get_drivers(force=self.force)
             df = self._model_drivers
 
         df = df.assign(is_driver=True)
@@ -2410,6 +2605,9 @@ class Model:
         return df
 
     def describe_drivers(self, filter_string=""):
+        """
+        Get a summary DataFrame of the model's drivers
+        """
         # if self.describe_drivers is not None:
         #    return self.describe_drivers
         # get the historical dataframe from candas
@@ -2437,6 +2635,9 @@ class Model:
         return df_drivers_summary
 
     def historical_data_frame(self):  # if cache = true save locally?
+        """
+        Get a DataFrame of the model's historical data
+        """
         if self.force == False:
             df = self.gt.get_file(self.ticker, "historical_data")
             if df is not None:
@@ -2543,9 +2744,7 @@ class Model:
 
         if self.csin == "":
             self.csin, self.latest_version = get_company_info_from_ticker(
-                self.ticker,
-                self.api_headers,
-                self.config.mds_host,
+                self.ticker, self.api_headers, self.config.mds_host
             )
 
         url = get_forecast_url(self.csin, self.latest_version, self.config.mds_host)
