@@ -257,7 +257,7 @@ def get_betas(x, y, n=0):
     return [beta, r2, n]
 
 
-def get_fred_data(series_list, config):
+def get_fred_data(series_list, config, quarters=True):
 
     end_date = datetime.today().strftime("%Y-%m-%d")
     FRED = Fred(config.fred_key)  # Fred(config.fred_key)
@@ -266,20 +266,22 @@ def get_fred_data(series_list, config):
     for fred_series in series_list:
         s = pd.DataFrame(
             FRED.get_series(
-                fred_series, observation_start="2014-09-02", observation_end=end_date
+                fred_series, observation_start="2016-01-01", observation_end=end_date
             )
         ).reset_index()
         s.columns = ["end_date", fred_series]
-        s = calendar_quarter(s, "end_date")
-        s = s.groupby("end_date_CALENDAR_QUARTER").last().reset_index()
+        s["quarter"] = pd.PeriodIndex(s.end_date, freq="Q")
+        # s = calendar_quarter(s, "end_date")
+        if quarters == True:
+            s = s.groupby("quarter").mean().reset_index()
         if df_fred.shape[0] > 0:
             s = s.drop(columns="end_date")
             df_fred = pd.merge(
                 df_fred,
                 s,
                 how="inner",
-                left_on="end_date_CALENDAR_QUARTER",
-                right_on="end_date_CALENDAR_QUARTER",
+                left_on="quarter",
+                right_on="quarter",
             )
         else:
             df_fred = s
@@ -341,7 +343,7 @@ def calendar_quarter(df, col, datetime=True):
     # translate a date into sort-able and group-able YYYY-mm format.
     df[col] = pd.to_datetime(df[col])
 
-    df[col + "shift"] = df[col] + pd.Timedelta(days=-12)
+    df[col + "shift"] = df[col] + pd.Timedelta(days=-5)
 
     df[col + "_CALENDAR_QUARTER"] = df[col + "shift"].dt.to_period("Q")
 
@@ -692,6 +694,7 @@ def regress_dataframe_groups(
     df_data, y_name="", return_grouped=True, n_periods=12, regression_warnings=False
 ):
     regression_warning = {}
+
     max_time_period = df_data["period_end_date"].max()
 
     period_names = list(
@@ -703,7 +706,6 @@ def regress_dataframe_groups(
     )
 
     warnings.filterwarnings("ignore", category=RuntimeWarning)
-
     df_data = df_data.sort_values(["time_series_name", "period_end_date"])
     df_data = df_data[df_data[y_name].notna()]
     df_data = df_data[df_data["value"].notna()]
