@@ -42,6 +42,9 @@ HISTORICAL_PERIODS_TO_FETCH = 24
 
 
 def set_api_key(key, key_file_path):
+    """
+    Loads API key from path
+    """
     with open(key_file_path) as f:
         keys_json = json.load(f)
 
@@ -62,6 +65,9 @@ def get_api_headers(canalyst_api_key: str) -> Dict[str, str]:
 
 
 class LogFile:
+    """
+    Logging helper class
+    """
 
     # to be refactored to log try: except: errors
 
@@ -94,6 +100,10 @@ class LogFile:
 
 # helper class to get data from S3
 class Getter:
+    """
+    S3 Client to retreive data from S3
+    """
+
     def __init__(self, config: Config):
         self.config = config
         self.log = LogFile(default_dir=self.config.default_dir)
@@ -104,6 +114,9 @@ class Getter:
         )
 
     def get_s3_file(self, to_filename, from_filename):
+        """
+        Download file from S3
+        """
         try:
             with open(to_filename, "wb+") as f:
                 self.s3_client.download_fileobj("candas", from_filename, f)
@@ -113,6 +126,9 @@ class Getter:
         return
 
     def get_file_from_s3(self, file_name):
+        """
+        Get file from S3
+        """
         try:
             csv_obj = self.s3_client.get_object(Bucket="candas", Key=file_name)
             body = csv_obj["Body"].read()
@@ -122,6 +138,9 @@ class Getter:
         return body
 
     def get_zip_csv_from_s3(self, file_name):
+        """
+        Get zip from S3
+        """
         d = self.s3_client.get_object(Bucket="candas", Key=file_name)
         import io
 
@@ -134,6 +153,9 @@ class Getter:
         return df
 
     def get_csv_from_s3(self, file_name):
+        """
+        Get CSV from S3
+        """
         try:
             csv_obj = self.s3_client.get_object(Bucket="candas", Key=file_name)
             body = csv_obj["Body"]
@@ -145,7 +167,11 @@ class Getter:
         return df
 
     def get_file(self, ticker, type):
+        """
+        Gets ticker file from S3
 
+        Uses local filesystem if no file found on S3
+        """
         file_ticker = ticker.split(" ")[0]
         path_name = f"DATA/{file_ticker}/{ticker}_{type}.csv"
         df = self.get_csv_from_s3(path_name)
@@ -167,6 +193,9 @@ class Getter:
 
 
 def save_guidance_csv(df, ticker, filename):
+    """
+    Save guidance CSV
+    """
     pathname = filename  # CWD + "\\DATA\\" + filename + "\\"  # .replace(r,'')
 
     for i, row in df.iterrows():
@@ -198,6 +227,7 @@ def filter_dataset(
     unit_type="",
     category="",
     warning=True,
+    n_diff="",
 ):
 
     if (
@@ -231,9 +261,6 @@ def filter_dataset(
     if is_driver != "":
         df = df.loc[df["is_driver"] == is_driver]
 
-    if period_name != "":
-        df = df.loc[df["period_name"] == period_name]
-
     if is_historical != "":
         df = df.loc[df["is_historical"] == is_historical]
 
@@ -283,6 +310,10 @@ def filter_dataset(
 
         df = df.sort_values("period_end_date")
 
+        if n_diff != "":
+            df = df.sort_values(["ticker", "period_end_date"])
+            df["value"] = df["value"].pct_change(periods=int(n_diff))
+
         if pivot is True:
             df = pivot_df(df, mrq_notation)
         return df
@@ -295,6 +326,10 @@ def filter_dataset(
             list_df.append(df_copy)
         df = pd.concat(list_df)
 
+        if n_diff != "":
+            df = df.sort_values(["ticker", "period_end_date"])
+            df["value"] = df["value"].pct_change(periods=int(n_diff))
+
     if pivot is True:
         df = pivot_df(df, mrq_notation)
 
@@ -304,7 +339,9 @@ def filter_dataset(
 def send_scenario(
     ticker, data, auth_headers, csin="", latest_version="", mds_host=settings.MDS_HOST
 ):
-
+    """
+    Makes a POST request for scenario to the MDS
+    """
     if csin == "" or latest_version == "":
         csin, latest_version = get_company_info_from_ticker(
             ticker, auth_headers, mds_host
@@ -325,6 +362,9 @@ def send_scenario(
 
 
 def make_scenario_data(feature_name, feature_period, feature_value, scenario_name):
+    """
+    Create JSON structured data from scenario information
+    """
     data = {
         "changes": [
             {
@@ -349,7 +389,7 @@ def map_scenario_urls(json):
     return scenario_map
 
 
-def get_drivers_from_model(
+def get_name_index_from_csv(
     ticker,
     config,
 ):
@@ -358,11 +398,11 @@ def get_drivers_from_model(
     # check if the model exists if so use it.
     file_ticker = ticker.split(" ")[0]
 
-    log = LogFile(default_dir=default_dir)
+    log = LogFile(default_dir=config.default_dir)
 
     get_excel_model(ticker, config, default_dir)
 
-    path_name = f"{default_dir}/DATA/{file_ticker}/"
+    path_name = f"{config.default_dir}/DATA/{file_ticker}/"
 
     cache_files = []
     _, files = scandir(path_name, [".xlsx"])
@@ -399,116 +439,8 @@ def get_drivers_from_model(
         df_index["index"] = df_index["index"].astype(int)
         df_index = df_index.sort_values("index")
 
-        for named_range in list_names:
-            if "LatestMRQ" in str(named_range):
-                mrq = str(named_range.value)
-                mrq = mrq.replace('"', "")
-            if "Common_ColumnHeader" in str(named_range):
-                col_hdr = str(named_range.value)
-                col_hdr = str(named_range.value).split("!")[1]
-                col_hdr = col_hdr.replace('"', "")
-
-        col_hdr = col_hdr.split(":")[0]
-        col_hdr = col_hdr.replace("$", "")
-        col_hdr = "A" + col_hdr + ":DA" + col_hdr
-
-        def n2a(n, b=string.ascii_uppercase):
-            d, m = divmod(n, len(b))
-            return n2a(d - 1, b) + b[m] if d else b[m]
-
-        driver_sheet = workbook["Drivers"]
-
-        cells = list(driver_sheet["A5":"DA5"])
-        headers = [cell.value for cell in cells[0]]
-        for index, item in enumerate(headers):
-            if item == mrq:
-                col_loc_i = index
-
-        # not 'z_' in str(named_range) and
-        named_ranges = {}
-        for named_range in list_names:
-            # and not 'z_' in str(named_range)
-            if (
-                "MO_" in str(named_range.name)
-                and ":" in str(named_range.attr_text)
-                and not "MO_Section" in str(named_range.name)
-                and not "MO_SPT_" in str(named_range.name)
-                and not "MO_SubSection_" in str(named_range.name)
-                and not "MO_SNA_" in str(named_range.name)
-                and not "MO_Checks_" in str(named_range.name)
-                and not "MO_Common_" in str(named_range.name)
-            ):
-                z = named_range.attr_text.split(":")
-                z[0] = z[0].split("!")[1]
-                named_ranges[named_range.name] = z[0]
-
-        list_df = []
-
-        for defined_name in named_ranges:
-            row = named_ranges[defined_name].replace("$", "")
-            try:
-                list_values = [
-                    val
-                    for val in model_sheet.iter_rows(min_row=int(row), max_row=int(row))
-                ][0]
-            except:
-                log.write(f"error: {defined_name}")
-                continue
-
-            dict_values = {}
-
-            for index, item in enumerate(list_values):
-                i_toggle = 0
-                if type(item.value) == float or type(item.value) == int:
-                    dict_values = {}
-                    num = (
-                        "$"
-                        + n2a(index)
-                        + str(named_ranges[defined_name]).replace("$", "")
-                    )
-                    font_color = item.font.color
-                    if font_color and font_color.type == "rgb":
-                        color = font_color.rgb
-                    elif font_color and font_color.type == "indexed":
-                        color = styles.colors.COLOR_INDEX[font_color.indexed]
-                    else:
-                        color = styles.colors.COLOR_INDEX[0]  # black
-
-                    if color.endswith("FF0000"):
-                        if index <= col_loc_i:
-                            dict_values["period_name"] = headers[index - 1]
-                            dict_values["cell"] = num
-                            dict_values["actual"] = 0
-                            dict_values["historical"] = 1
-                        else:
-                            dict_values["period_name"] = headers[index - 1]
-                            dict_values["cell"] = num
-                            dict_values["actual"] = 0
-                            dict_values["historical"] = 0
-                        i_toggle = 1
-                    else:
-                        if index <= col_loc_i:
-                            dict_values["period_name"] = headers[index - 1]
-                            dict_values["cell"] = num
-                            dict_values["actual"] = 1
-                            dict_values["historical"] = 1
-                        else:
-                            dict_values["period_name"] = headers[index - 1]
-                            dict_values["cell"] = num
-                            dict_values["actual"] = 1
-                            dict_values["historical"] = 0
-                    df = pd.DataFrame(dict_values, index=[0])
-                    df["named_range"] = defined_name
-
-                    if i_toggle == 1:
-                        list_df.append(df)
-
-        df = pd.concat(list_df).sort_values(["named_range", "cell"])
-        df["filename"] = f_path
-
         workbook.close()
-        list_for.append(df)
-    return pd.concat(list_for), df_index
+    return df_index
 
 
 def get_scenarios(ticker, auth_headers, mds_host=settings.MDS_HOST):
@@ -531,6 +463,11 @@ def get_scenarios(ticker, auth_headers, mds_host=settings.MDS_HOST):
 
 
 def post_request(url, headers, json):
+    """
+    POST request helper function
+
+    Uses timer to account for throttle limits.
+    """
     response = requests.post(url=url, headers=headers, json=json, verify=False)
     if check_throttle_limit(response):
         response = requests.post(url=url, headers=headers, json=json, verify=False)
@@ -539,9 +476,12 @@ def post_request(url, headers, json):
 
 
 def get_company_info_from_ticker(ticker, auth_headers, mds_host=settings.MDS_HOST):
-
+    """
+    Get CSIN and latest version for a ticker
+    """
     company_url = f"{mds_host}/{CSIN_URL.format(ticker=quote_plus(ticker))}"
     response = requests.get(company_url, headers=auth_headers, verify=False)
+
     json = response.json().get("results")[0]
     csin = json.get("csin")
     latest_version = (
@@ -552,6 +492,9 @@ def get_company_info_from_ticker(ticker, auth_headers, mds_host=settings.MDS_HOS
 
 
 def get_csin_from_ticker(ticker, auth_headers, mds_host=settings.MDS_HOST):
+    """
+    Get CSIN for ticker
+    """
     company_url = f"{mds_host}/{CSIN_URL.format(ticker=quote_plus(ticker))}"
     response = requests.get(company_url, headers=auth_headers, verify=False)
     json = response.json().get("results")[0]
@@ -560,6 +503,9 @@ def get_csin_from_ticker(ticker, auth_headers, mds_host=settings.MDS_HOST):
 
 
 def get_excel_model(ticker, config, download_dir=""):
+    """
+    Retrieve Excel model from Web Portal
+    """
     auth_headers = get_api_headers(config.canalyst_api_key)
     mds_host = config.mds_host
     wp_host = config.wp_host
@@ -622,6 +568,9 @@ def get_excel_model(ticker, config, download_dir=""):
 
 
 def create_pdf_from_dot(dot_file):
+    """
+    Create a PDF file from a Dot file
+    """
     pdf_file = dot_file.replace(".dot", ".pdf")
     import graphviz
 
@@ -631,6 +580,9 @@ def create_pdf_from_dot(dot_file):
 
 
 def get_forecast_url(csin, latest_version, mds_host=settings.MDS_HOST):
+    """
+    Return a formatted MDS forecast endpoint
+    """
     return (
         f"{mds_host}/api/equity-model-series/{csin}/equity-models/"
         f"{latest_version}/forecast-periods/"
@@ -787,10 +739,7 @@ def get_forecast_url_data(res_dict, ticker, auth_headers):
 
 
 def get_scenario_url_data(
-    res_dict,
-    ticker,
-    auth_headers,
-    default_dir=settings.DEFAULT_DIR,
+    res_dict, ticker, auth_headers, default_dir=settings.DEFAULT_DIR
 ):
     log = LogFile(default_dir=default_dir)
     url = res_dict["self"]
@@ -847,6 +796,11 @@ def get_scenario_url_data(
 
 
 def get_request(url, headers):
+    """
+    Wrapper for a GET request
+
+    Handles throttle limits by using a timer
+    """
     response = requests.get(url=url, headers=headers, verify=False)
     if check_throttle_limit(response):
         response = requests.get(url=url, headers=headers, verify=False)
@@ -854,6 +808,9 @@ def get_request(url, headers):
 
 
 def check_throttle_limit(response):
+    """
+    Checks to see if a throttle limit has been hit, waits until throttle is lifted.
+    """
     if response.status_code == 429:
         timer = int(response.headers.get("Retry-After"))
         print(f"Please wait...Retrying request in {timer} seconds")
@@ -943,6 +900,9 @@ def create_drivers_dot(time_series, auth_headers, ticker, config=None, s3_client
 
 
 def get_model_url(ticker, wp_host=settings.WP_HOST):
+    """
+    Return MDS endpoint URL for model
+    """
     file_ticker = ticker.split(" ")[0]
     url = f"{wp_host}/files/search?query={file_ticker}"
     return url
@@ -950,11 +910,11 @@ def get_model_url(ticker, wp_host=settings.WP_HOST):
 
 # retrieve latest equity model from API
 def get_model_info(
-    ticker,
-    auth_headers,
-    default_dir=settings.DEFAULT_DIR,
-    mds_host=settings.MDS_HOST,
+    ticker, auth_headers, default_dir=settings.DEFAULT_DIR, mds_host=settings.MDS_HOST
 ):
+    """
+    Retrieve latest equity model information and historical datapoints endpoint url
+    """
     log = LogFile(default_dir=default_dir)
     model_info = {}
 
@@ -996,6 +956,9 @@ def get_model_info(
 
 
 def write_json(json_data, file_name):
+    """
+    Write JSON to file
+    """
     with open(file_name, "w") as f:
         json.dump(json_data, f)
 
@@ -1008,6 +971,9 @@ def crawl_company_pages(
     auth_headers,
     default_dir=settings.DEFAULT_DIR,
 ):
+    """
+    Retreive data from url and subsequent pages.
+    """
     file_ticker = ticker.split(" ")[0]
 
     page_number = 1
@@ -1024,6 +990,11 @@ def crawl_company_pages(
 
 
 def get_json(url, key_name, next_name, auth_headers):
+    """
+    Make a GET request
+
+    Returns (response, next_url)
+    """
     response = requests.get(url, headers=auth_headers, verify=False)
     res = response.json().get(key_name)
     next_url = response.json().get(next_name)
@@ -1046,6 +1017,9 @@ def read_json(ticker, default_dir=settings.DEFAULT_DIR):
 
 
 def json_to_df(dict_json, ticker):
+    """
+    Creates a dataframe out of JSON data
+    """
     json_key_list = sorted(dict_json.keys())
     list_out = []
     for key in json_key_list:
@@ -1161,6 +1135,9 @@ def pivot_df(df, mrq_notation):
 
 
 def refresh_cache(config: Config = None, log: LogFile = None, s3_client: Getter = None):
+    """
+    Refresh the cache by deleting and pulling files from S3
+    """
     if not config:
         config = settings.CONFIG or resolve_config()
     if not log:
@@ -1221,6 +1198,9 @@ def get_candas_ticker_list(ticker="", config=None):
 
 
 def find_in_list(feature_list, search_term):
+    """
+    Finds search term in items from passed in list
+    """
     return_list = []
     for item in feature_list:
         if search_term.lower() in item.lower():
@@ -1285,6 +1265,64 @@ def calendar_quarter(df, col, datetime=True):
     return df
 
 
+def get_drivers_from_api(mds_host, csin, model_version, api_headers):
+    """
+    Returns a list of drivers for the model with the specified CSIN and model version
+    """
+    driver_list = []
+    logger = LogFile(default_dir=settings.CONFIG.default_dir)
+
+    forecast_period_name = get_forecast_period_name(
+        mds_host, csin, model_version, api_headers, logger
+    )
+
+    driver_url = "api/equity-model-series/{csin}/equity-models/{model_version}/forecast-periods/{period}/data-points/?is_driver=true&page_size=200"
+    driver_endpoint = f"{mds_host}/{driver_url.format(csin=quote_plus(csin), model_version=quote_plus(model_version), period=quote_plus(forecast_period_name))}"
+    next_url = driver_endpoint
+
+    while next_url is not None:
+        drivers, next_url = _get_drivers_from_next_url(next_url, api_headers, logger)
+        driver_list.extend(drivers)
+
+    return driver_list
+
+
+def get_forecast_period_name(mds_host, csin, model_version, api_headers, logger):
+    """
+    Retrieve the first forecast period name
+    """
+    forecast_url = (
+        "api/equity-model-series/{csin}/equity-models/{model_version}/forecast-periods/"
+    )
+    forecast_endpoint = f"{mds_host}/{forecast_url.format(csin=quote_plus(csin), model_version=quote_plus(model_version))}"
+    try:
+        response = requests.get(forecast_endpoint, headers=api_headers)
+    except:
+        logger.write(f"Candas: Error with getting forecast period.")
+        return
+
+    forecast_period_name = response.json()["results"][0]["name"]
+
+    return forecast_period_name
+
+
+def _get_drivers_from_next_url(url, api_headers, logger):
+    """
+    Get drivers from specified url
+    """
+    try:
+        response = requests.get(url, headers=api_headers)
+    except:
+        logger.write(f"Candas: Error with getting drivers.")
+        return
+
+    response_json = response.json()
+    drivers = response_json["results"]
+    next_url = response_json["next"]
+
+    return drivers, next_url
+
+
 COMP_LISTS = {}
 COMP_LISTS["Apparel"] = [
     "FL US",
